package org.fuseri.modulelanguageschool.course;

import jakarta.validation.Valid;
import jakarta.validation.constraints.PositiveOrZero;
import org.fuseri.model.dto.course.CourseCreateDto;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.UserDto;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a RESTful controller for Courses resources.
 * It handles incoming HTTP requests related to courses, and delegates them to the appropriate service method.
 */
@RestController
@RequestMapping("/courses")
public class CourseController {

    public static final String COURSE_NAME = "english b2 course";

    /**
     * Creates a new course.
     *
     * @param dto the CourseCreateDto containing the course data
     * @return the newly created CourseDto
     */
    @PostMapping("/create")
    public CourseDto create(@Valid @RequestBody CourseCreateDto dto) {
        return new CourseDto(dto.getName(), dto.getCapacity(), dto.getLanguageTypeDto(), dto.getProficiencyLevelDto());
    }

    /**
     * Retrieves a course by ID.
     *
     * @param id the ID of the course to retrieve
     * @return the CourseDto for the specified ID
     */
    @GetMapping("/find/{id}")
    public CourseDto find(@PathVariable String id) {
        return new CourseDto(COURSE_NAME, 10, LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2);
    }

    /**
     * Retrieves a paginated list of courses
     *
     * @param page the page number to retrieve
     * @return the Result containing the requested page of CourseDtos
     */
    @GetMapping("/findAll")
    public List<CourseDto> findAll(@RequestParam int page) {
        return new ArrayList<>();
    }

    /**
     * Retrieves a paginated list of courses of a given language
     *
     * @param page the page number to retrieve
     * @param lang the language to find courses of
     * @return the Result containing the requested page of CourseDtos
     */
    @GetMapping("/findAllByLang")
    public List<CourseDto> findAll(@RequestParam int page, @RequestParam LanguageTypeDto lang) {
        return new ArrayList<>();
    }

    /**
     * Retrieves a paginated list of courses of a given language and proficiency
     *
     * @param page the page number to retrieve
     * @param lang the language to find courses of
     * @param prof the proficiency of the language
     * @return the Result containing the requested page of CourseDtos
     */
    @GetMapping("/findAllByLangProf")
    public List<CourseDto> findAll(@RequestParam int page,
                                   @RequestParam LanguageTypeDto lang,
                                   @RequestParam ProficiencyLevelDto prof) {
        return new ArrayList<>();
    }

    /**
     * Updates an existing course.
     *
     * @param id  the ID of the course to update
     * @param dto the CourseCreateDto containing the updated course data
     * @return the updated CourseDto
     */
    @PutMapping("/update/{id}")
    public CourseDto update(@PathVariable String id, @Valid @RequestBody CourseCreateDto dto) {
        return new CourseDto(dto.getName(), dto.getCapacity(), dto.getLanguageTypeDto(), dto.getProficiencyLevelDto());
    }

    /**
     * Deletes a course by ID.
     *
     * @param id the ID of the course to delete
     * @return true if the course was successfully deleted, false otherwise
     */
    @DeleteMapping("/delete/{id}")
    public void delete(@PathVariable String id) {
    }


    /**
     * Adds student to the existing course
     *
     * @param id      id of course to update
     * @param student UserDto for the student
     * @return the CourseDto representing the updated course
     */
    @PatchMapping("/enrol/{id}")
    public CourseDto enrol(@PathVariable String id, @RequestBody UserDto student) {
        var course = new CourseDto(COURSE_NAME, 10, LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2);
        course.setStudentIds(new ArrayList<>(List.of(student.getId())));
        return course;
    }

    /**
     * Removes student from the existing course
     *
     * @param id      id of lecture to update
     * @param student UserDto for the student
     * @return the CourseDto representing the updated course
     */
    @PatchMapping("/expel/{id}")
    public CourseDto expel(@PathVariable String id, @RequestBody UserDto student) {
        return new CourseDto(COURSE_NAME, 10, LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2);
    }

}
