package org.fuseri.modulelanguageschool.lecture;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import org.fuseri.model.dto.lecture.LectureCreateDto;
import org.fuseri.model.dto.lecture.LectureDto;
import org.fuseri.model.dto.user.AddressDto;
import org.fuseri.model.dto.user.UserDto;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import java.time.LocalDateTime;
import java.util.ArrayList;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
public class LectureTest {
    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private LectureController lectureController;

    private final LectureCreateDto lectureCreateDto = new LectureCreateDto(
            LocalDateTime.now().plusDays(2),
            LocalDateTime.now().plusDays(2).plusHours(2),
            "Learning how to spell deprecated",
            10, "0");
    private final LectureDto lectureDto = new LectureDto(
            LocalDateTime.now().plusDays(2),
            LocalDateTime.now().plusDays(2).plusHours(2),
            "Learning how to spell deprecated",
            10, "0", "0");

    @Test
    void createLecture() throws Exception {
        Mockito.when(lectureController.create(ArgumentMatchers.isA(LectureCreateDto.class))).thenReturn(lectureDto);
        mockMvc.perform(post("/lectures/create")
                        .content(asJsonString(lectureCreateDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.from").isNotEmpty())
                .andExpect(jsonPath("$.to").isNotEmpty())
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value("10"))
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void createInvalidLecture() throws Exception {
        LectureCreateDto invalidLectureCreateDto =
                new LectureCreateDto(null, null, null, null, null);
        Mockito.when(lectureController.create(ArgumentMatchers.isA(LectureCreateDto.class))).thenReturn(lectureDto);
        mockMvc.perform(post("/lectures/create")
                        .content(asJsonString(invalidLectureCreateDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void createLectureWithoutParameter() throws Exception {
        Mockito.when(lectureController.create(ArgumentMatchers.isA(LectureCreateDto.class))).thenReturn(lectureDto);
        mockMvc.perform(post("/lectures/create"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findLecture() throws Exception {
        String id = "0";
        Mockito.when(lectureController.find(id)).thenReturn(lectureDto);
        mockMvc.perform(get("/lectures/find/" + id))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.from").isNotEmpty())
                .andExpect(jsonPath("$.to").isNotEmpty())
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value("10"))
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void findLectureWithoutId() throws Exception {
        Mockito.when(lectureController.find(ArgumentMatchers.anyString())).thenReturn(lectureDto);
        mockMvc.perform(get("/lectures/find/"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findLecturesByCourse() throws Exception {
        String id = "0";
        Mockito.when(lectureController.findByCourse(id)).thenReturn(new ArrayList<>());
        String response = mockMvc.perform(get("/lectures/findByCourse")
                        .param("courseId", id))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertThat("response", response, is("[]"));
    }

    @Test
    void findLecturesByCourseWithoutParameter() throws Exception {
        Mockito.when(lectureController.findByCourse(ArgumentMatchers.anyString())).thenReturn(new ArrayList<>());
        mockMvc.perform(get("/lectures/findByCourse"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void updateLecture() throws Exception {
        String id = "0";
        Mockito.when(lectureController.update(ArgumentMatchers.eq(id),
                ArgumentMatchers.isA(LectureCreateDto.class)))
                .thenReturn(lectureDto);
        mockMvc.perform(put("/lectures/update/" + id)
                        .content(asJsonString(lectureCreateDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.from").isNotEmpty())
                .andExpect(jsonPath("$.to").isNotEmpty())
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value("10"))
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void updateLectureWithoutParameter() throws Exception {
        Mockito.when(lectureController.
                update(ArgumentMatchers.anyString(), ArgumentMatchers.isA(LectureCreateDto.class)))
                .thenReturn(lectureDto);
        mockMvc.perform(put("/lectures/update"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void deleteLecture() throws Exception {
        String id = "0";
        Mockito.doNothing().when(lectureController).delete(id);
        mockMvc.perform(delete("/lectures/delete/" + id))
                        .andExpect(status().isOk());
    }

    @Test
    void deleteCourseWithoutParameter() throws Exception {
        Mockito.doNothing().when(lectureController).delete(ArgumentMatchers.anyString());
        mockMvc.perform(delete("/lectures/delete/"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void setLecturerForLecture() throws Exception {
        String id = "0";
        Mockito.when(lectureController.setLecturer(ArgumentMatchers.eq(id),
                ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(lectureDto);
        UserDto student = new UserDto("novakovat","novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());
        mockMvc.perform(patch("/lectures/setLecturer/" + id)
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.from").isNotEmpty())
                .andExpect(jsonPath("$.to").isNotEmpty())
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value("10"))
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void setLecturerForLectureWithoutParameters() throws Exception {
        Mockito.when(lectureController.setLecturer(ArgumentMatchers.anyString(),
                        ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(lectureDto);
        mockMvc.perform(patch("/lectures/setLecturer"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void enrolLecture() throws Exception {
        String id = "0";
        Mockito.when(lectureController.enrol(ArgumentMatchers.eq(id),
                ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(lectureDto);
        UserDto student = new UserDto("novakovat","novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());
        mockMvc.perform(patch("/lectures/enrol/" + id)
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.from").isNotEmpty())
                .andExpect(jsonPath("$.to").isNotEmpty())
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value("10"))
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void enrolCourseWithoutUserParameters() throws Exception {
        Mockito.when(lectureController.enrol(ArgumentMatchers.anyString(),
                ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(lectureDto);
        mockMvc.perform(patch("/lectures/enrol"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void expelLecture() throws Exception {
        String id = "0";
        Mockito.when(lectureController.expel(ArgumentMatchers.eq(id),
                ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(lectureDto);
        UserDto student = new UserDto("novakovat","novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());
        mockMvc.perform(patch("/lectures/expel/" + id)
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.from").isNotEmpty())
                .andExpect(jsonPath("$.to").isNotEmpty())
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value("10"))
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void expelCourseWithoutUserParameters() throws Exception {
        Mockito.when(lectureController.expel(ArgumentMatchers.anyString(),
                ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(lectureDto);
        mockMvc.perform(patch("/lectures/expel"))
                .andExpect(status().is4xxClientError());
    }

    public static String asJsonString(final Object obj) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            mapper.registerModule(new JavaTimeModule());
            return mapper.writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
