package org.fuseri.modulelanguageschool.user;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.*;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;


@SpringBootTest
@AutoConfigureMockMvc
class UserControllerTest {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private MockMvc mockMvc;

    private static final AddressDto ADDRESS_TO_CREATE = new AddressDto(
            "Czechia", "Brno", "Masarykova", "45", "90033");

    private static final List<AddressDto> INVALID_ADDRESSES = List.of(
            new AddressDto("", "Brno", "Masarykova", "45", "90033"),
            new AddressDto("Czechia", "", "Masarykova", "45", "90033"),
            new AddressDto("Czechia", "Brno", "", "45", "90033"),
            new AddressDto("Czechia", "Brno", "Masarykova", "", "90033"),
            new AddressDto("Czechia", "Brno", "Masarykova", "45", ""),
            new AddressDto(null, "Brno", "Masarykova", "45", "90033"),
            new AddressDto("Czechia", null, "Masarykova", "45", "90033"),
            new AddressDto("Czechia", "Brno", null, "45", "90033"),
            new AddressDto("Czechia", "Brno", "Masarykova", null, "90033")
    );

    private static final UserCreateDto USER_TO_CREATE = new UserCreateDto(
            "xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
            "xnovak@emample.com", "Peter", "Novak", ADDRESS_TO_CREATE);

    private static final UserLoginDto USER_TO_LOGIN = new UserLoginDto(
            "xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af");

    private static Stream<UserCreateDto> invalidUsers() {
        var invalidUsers = Stream.of(
                new UserCreateDto("", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", "Peter", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "",
                        "xnovak@emample.com", "Peter", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "", "Peter", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", "", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", "Peter", "", ADDRESS_TO_CREATE),
                new UserCreateDto(null, "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", "Peter", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", null,
                        "xnovak@emample.com", "Peter", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        null, "Peter", "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", null, "Novak", ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", "Peter", null, ADDRESS_TO_CREATE),
                new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                        "xnovak@emample.com", "Peter", "Novak", null)
        );

        var invalidAddressUsers = new ArrayList<UserCreateDto>();
        for (var invalidAddress : INVALID_ADDRESSES) {
            invalidAddressUsers.add(new UserCreateDto("xnovak", "1c1bbf66-6585-4978-886b-b126335ff3af",
                    "xnovak@emample.com", "Peter", "Novak", invalidAddress));
        }

        return Stream.concat(invalidUsers, invalidAddressUsers.stream());
    }

    private static Stream<UserLoginDto> invalidLoginDtoStream() {
        return Stream.of(
                new UserLoginDto("", "1c1bbf66-6585-4978-886b-b126335ff3af"),
                new UserLoginDto("xnovak", ""),
                new UserLoginDto(null, "1c1bbf66-6585-4978-886b-b126335ff3af"),
                new UserLoginDto("xnovak", null));
    }

    @Test
    void create() throws Exception {
        mockMvc.perform(post("/users")
                        .content(asJsonString(USER_TO_CREATE))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }

    @ParameterizedTest
    @MethodSource("invalidUsers")
    void createInvalidUser(UserCreateDto user) throws Exception {
        mockMvc.perform(post("/users")
                        .content(asJsonString(user))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findUser() throws Exception {
        String response = mockMvc.perform(post("/users")
                        .content(asJsonString(USER_TO_CREATE))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString();

        String id = objectMapper.readValue(response, UserDto.class).getId();

        mockMvc.perform(get("/users/{id}", id))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id").value(id));
    }

    @Test
    void findAll() throws Exception {
        mockMvc.perform(get("/users/all")
                        .param("page", "0"))
                .andExpect(status().isOk());
    }

    @Test
    void deleteUser() throws Exception {
        String response = mockMvc.perform(post("/users")
                        .content(asJsonString(USER_TO_CREATE))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString();

        String id = objectMapper.readValue(response, UserDto.class).getId();

        mockMvc.perform(delete("/users/{id}", id)
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }

    @Test
    void update() throws Exception {
        String response = mockMvc.perform(post("/users")
                        .content(asJsonString(USER_TO_CREATE))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString();

        String id = objectMapper.readValue(response, UserDto.class).getId();

        var updatedUsername = "novak";
        var userToUpdate = new UserCreateDto(
                USER_TO_CREATE.getUsername(), USER_TO_CREATE.getPassword(), USER_TO_CREATE.getEmail(),
                USER_TO_CREATE.getFirstName(), USER_TO_CREATE.getLastName(), USER_TO_CREATE.getAddress());
        userToUpdate.setUsername(updatedUsername);

        mockMvc.perform(put("/users/update/{id}", id)
                        .content(asJsonString(userToUpdate))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.username").value(updatedUsername));
    }

    @Test
    void login() throws Exception {
        mockMvc.perform(post("/users/login")
                        .content(asJsonString(USER_TO_LOGIN))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }

    @ParameterizedTest
    @MethodSource("invalidLoginDtoStream")
    void loginInvalidDto(UserLoginDto loginDto) throws Exception {
        mockMvc.perform(post("/users/login")
                        .content(asJsonString(loginDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void logout() throws Exception {
        String response = mockMvc.perform(post("/users")
                        .content(asJsonString(USER_TO_CREATE))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString();

        String id = objectMapper.readValue(response, UserDto.class).getId();

        mockMvc.perform(post("/users/logout/{id}", id))
                .andExpect(status().isOk());
    }

    @Test
    void getFinished() throws Exception {
        mockMvc.perform(get("/users/finished/{id}", "1c1bbf66-6585-4978-886b-b126335ff3af"))
                .andExpect(status().isOk());
    }

    @Test
    void getEnrolled() throws Exception {
        mockMvc.perform(get("/users/enrolled/{id}", "1c1bbf66-6585-4978-886b-b126335ff3af"))
                .andExpect(status().isOk());
    }

    @Test
    void addLanguage() throws Exception {
        mockMvc.perform(put("/users/addLanguage/{id}", "1c1bbf66-6585-4978-886b-b126335ff3af")
                        .content(asJsonString(new UserAddLanguageDto(LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }

    private static String asJsonString(final Object obj) throws JsonProcessingException {
        return new ObjectMapper().writeValueAsString(obj);
    }
}